#include "precompile.h"

#include <google/protobuf/message.h>
#include <a8/websocketsession.h>
#include <a8/tcplistener.h>

#include "framework/cpp/netmsghandler.h"
#include "framework/cpp/im_msgid.pb.h"
#include "framework/cpp/im_proto.pb.h"

#include "app.h"
#include "GCListener.h"
#include "jsondatamgr.h"
#include "ss_proto.pb.h"
#include "handlermgr.h"
#include "gameclientmgr.h"
#if MASTER_MODE
#include "mastersvrmgr.h"
#endif

class GCClientSession: public a8::WebSocketSession
{
public:

    virtual void DecodeUserPacket(char* buf, int& offset, unsigned int buflen) override
    {
        //packagelen + msgid + magiccode + msgbody
        //2 + 2 + 4+ xx + \0 + xx
        bool warning = false;
        while (buflen - offset >= sizeof(f8::PackHead)) {
            f8::PackHead* p = (f8::PackHead*)&buf[offset];
            if (p->magic_code == f8::MAGIC_CODE) {
                if (buflen - offset < sizeof(f8::PackHead) + p->packlen) {
                    break;
                }
                if (p->msgid > im::MaxIMMsgId) {
                    App::Instance()->AddSocketMsg(SF_Client,
                                                  socket_handle,
                                                  saddr,
                                                  p->msgid,
                                                  p->seqid,
                                                  &buf[offset + sizeof(f8::PackHead)],
                                                  p->packlen);
                }
                offset += sizeof(f8::PackHead) + p->packlen;
            } else {
                warning = true;
                offset++;
                continue;
            }
        }

        if (warning) {
            a8::UdpLog::Instance()->Warning("收到client非法数据包", {});
        }
    }

    virtual void OnRawHttpGet(const std::string& url, const std::string& querystr,
                              std::string& response) override
    {
        App::Instance()->AddIMMsg(IM_ExecGM,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(url)
                                  .SetParam2(querystr)
                                  .SetParam3(saddr));
    }

    virtual bool HandleRedirect(const std::string& url, const std::string& querystr,
                                std::string& location) override
    {
        im::IMSocketConnect msg;
        msg.set_is_websocket(true);
        msg.set_url(url);
        msg.set_query_str(querystr);

        std::string data;
        msg.SerializeToString(&data);
        App::Instance()->AddSocketMsg(SF_Client,
                                      socket_handle,
                                      saddr,
                                      im::_IMSocketConnect,
                                      0,
                                      data.data(),
                                      data.size());
#if MASTER_MODE
        a8::HTTPRequest request;
        a8::ParserUrlQueryString(querystr.c_str(), request);
        if (a8::Get(request, "c").GetString() == "Ops" &&
            a8::Get(request, "a").GetString() == "join") {
            std::string team_uuid = a8::Get(request, "team_uuid").GetString();
            std::vector<std::string> strings;
            a8::Split(team_uuid, strings, '_');
            if (strings.size() >= 2) {
                int node_id = a8::XValue(strings[0]);
                if (node_id != App::Instance()->node_id) {
                    std::string host;
                    if (JsonDataMgr::Instance()->GetNodeHost(node_id, host)) {
                        location = a8::Format("wss://%s/webapp/index.php?c=Ops&a=join&team_uuid=%s",
                                              {
                                                  host,
                                                  team_uuid
                                              });
                        return true;
                    }
                }
            }
        }
#endif
        return false;
    }

    virtual void OnDisConnect() override
    {
        im::IMSocketDisconnect msg;
        std::string data;
        msg.SerializeToString(&data);
        App::Instance()->AddSocketMsg(SF_Client,
                                      socket_handle,
                                      saddr,
                                      im::_IMSocketDisconnect,
                                      0,
                                      data.data(),
                                      data.size());
    }

};

static void CreateGameClientSocket(a8::TcpSession **p)
{
    *p = new GCClientSession();
}

static void GSListeneron_error(a8::TcpListener*, int type, int errorid)
{
    a8::UdpLog::Instance()->Debug("GCListeneron_error %d %d", {type, errorid});
}

void GCListener::Init()
{
    tcp_listener_ = new a8::TcpListener();
    tcp_listener_->on_create_client_socket = CreateGameClientSocket;
    tcp_listener_->on_error = GSListeneron_error;

    tcp_listener_->bind_address = "0.0.0.0";
    tcp_listener_->bind_port = JsonDataMgr::Instance()->GetConf()->At("listen_port")->AsXValue();
    tcp_listener_->Open();
}

void GCListener::UnInit()
{
    delete tcp_listener_;
    tcp_listener_ = nullptr;
}

void GCListener::_F8_IMSocketConnect(f8::MsgHdr& hdr, const im::IMSocketConnect& msg)
{
    if (msg.url().size() > 1024) {
        a8::UdpLog::Instance()->Warning("websokcet connect url to long %s %s",
                                        {
                                         msg.url(),
                                         msg.query_str()
                                        });
        return;
    }
    if (msg.query_str().size() > 1024) {
        a8::UdpLog::Instance()->Warning("websokcet connect query_str to long %s %s",
                                        {
                                         msg.url(),
                                         msg.query_str()
                                        });
        return;
    }
    websocket_url_hash_[hdr.socket_handle] = std::make_tuple(msg.url(), msg.query_str());
}

void GCListener::_F8_IMSocketDisconnect(f8::MsgHdr& hdr, const im::IMSocketDisconnect& msg)
{
    GameClientMgr::Instance()->OnClientDisconnect(hdr.socket_handle);
#if MASTER_MODE
    MasterSvrMgr::Instance()->RemoveRequestBySocketHandle(hdr.socket_handle, true);
#endif
    websocket_url_hash_.erase(hdr.socket_handle);
}

void GCListener::ForwardTargetConnMsg(f8::MsgHdr& hdr)
{
    char* buff = (char*)malloc(sizeof(f8::PackHead) + hdr.buflen);
    f8::PackHead* head = (f8::PackHead*)buff;
    head->packlen = hdr.buflen;
    head->msgid = hdr.msgid;
    head->seqid = hdr.seqid;
    head->magic_code = f8::MAGIC_CODE;
    head->ext_len = hdr.buflen >> 16;
    if (hdr.buflen > 0) {
        memmove(buff + sizeof(f8::PackHead), hdr.buf, hdr.buflen);
    }

    tcp_listener_->SendClientMsg(hdr.socket_handle, buff, sizeof(f8::PackHead) + head->packlen);
    free(buff);
}

void GCListener::SendText(unsigned short sockhandle, const std::string& text)
{
    tcp_listener_->SendClientMsg(sockhandle, text.data(), text.size());
}

void GCListener::ForceCloseClient(unsigned short sockhandle)
{
    tcp_listener_->ForceCloseClient(sockhandle);
}

void GCListener::MarkClient(unsigned short sockhandle, bool is_active)
{
    tcp_listener_->MarkClient(sockhandle, is_active);
}

long long GCListener::GetSendNodeNum()
{
    return tcp_listener_->send_node_num;
}

long long GCListener::GetSentBytesNum()
{
    return tcp_listener_->sent_bytes_num;
}

bool GCListener::GetWebSocketUrl(int socket_handle, std::string& url, std::string& query_str)
{
    auto itr = websocket_url_hash_.find(socket_handle);
    if (itr != websocket_url_hash_.end()) {
        url = std::get<0>(itr->second);
        query_str = std::get<1>(itr->second);
        return true;
    } else {
        return false;
    }
}
