#include "precompile.h"

#include <a8/mutable_xobject.h>

#include "handlermgr.h"

#include "GCListener.h"
#include "mastersvrmgr.h"
#include "app.h"

#include "ss_proto.pb.h"

#include "framework/cpp/im_msgid.pb.h"
#include "framework/cpp/im_proto.pb.h"

static void _GMOpsSelfChecking(f8::JsonHttpRequest* request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("healthy", 1);
    request->resp_xobj->SetVal("max_rundelay", 10);
}

static void _GMOpsGetNodeId(f8::JsonHttpRequest* request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("node_id", App::Instance()->node_id);
}

void HandlerMgr::Init()
{
    RegisterNetMsgHandlers();
    RegisterGMMsgHandler("Ops$selfChecking", _GMOpsSelfChecking);
    RegisterGMMsgHandler("Ops$getNodeId", _GMOpsGetNodeId);
}

void HandlerMgr::UnInit()
{
}

void HandlerMgr::RegisterNetMsgHandlers()
{
    RegisterNetMsgHandler(&immsghandler, &GCListener::_F8_IMSocketConnect);
    RegisterNetMsgHandler(&immsghandler, &GCListener::_F8_IMSocketDisconnect);

    RegisterNetMsgHandler(&msmsghandler, &MasterSvrMgr::_SS_MS_ResponseTargetServer);
}

void HandlerMgr::ProcGMMsg(unsigned long saddr, int sockhandle,
                           const std::string& url, const std::string& querystr)
{
    if (url != "/webapp/index.php") {
        GCListener::Instance()->SendText(sockhandle, a8::HttpResponse(404, ""));
        return;
    }

    a8::HTTPRequest request;
    a8::ParserUrlQueryString(querystr.c_str(), request);

    std::string msgname = a8::Get(request, "c").GetString() + "$" + a8::Get(request, "a").GetString();
    auto itr = gmhandlers_.find(msgname);
    if (itr != gmhandlers_.end()) {
        f8::JsonHttpRequest* request = new f8::JsonHttpRequest;
        request->saddr = saddr;
        request->socket_handle = sockhandle;
        request->query_str = querystr;
        request->request.ReadFromUrlQueryString(querystr);
        itr->second(request);

        if (!request->pending){
            std::string response;
            request->resp_xobj->ToJsonStr(response);
            GCListener::Instance()->SendText(sockhandle, a8::HttpResponse(response));

            delete request;
        }
    } else {
        GCListener::Instance()->SendText(sockhandle, a8::HttpResponse("{}"));
    }
}

void HandlerMgr::RegisterGMMsgHandler(const std::string& msgname,
                                      void (*handler)(f8::JsonHttpRequest*))
{
    gmhandlers_[msgname] = handler;
}
